'*******************************************************************************
'* Programa PORTAOAUTO_ARNEBASIC2K_v2a.BAS
'* Desenvolvido por: Marcio Jose Soares
'
'* Verso original: PORTAOAUTO_ARNEBASIC2K.BAS - 16/08/2010
'* Verso atual: 16/05/2012
'* Arne Basic 2k
'*
'* Obs.: Este programa foi desenvolvido para controle de um motor AC ou DC
'* aplicado a um porto automtico para garagens residnciais ou outras.
'*
'* S1  - chave final de curso "porto aberto"
'* S2  - chave final de curso "porto fechado"
'* RL1 - rel para ligar motor
'* RL2 - rel para inverter motor
'* LP1, LP2 - Sada coletor aberto para lmpadas 12V - aviso "entrada/sada veculo"
'* LPH - sada coletor aberto para alerta - "problemas na garagem" (opcional)
'*
'* O motor utilizado pode ser AC ou DC. Veja no artigo o esquema de ligao
'* dos mesmos
'*
'*******************************************************************************

'*******************************************************************************
'Configs necessrios ao microcontrolador
$crystal = 10000000
$regfile = "attiny2313.dat"

'Config para pinos de I/O
Config Portb = &B00001111               'msb - entrada; lsb - sada
Config Portd = &B11100010               'PD0, PD2, PD3 e PD4 - entrada, o resto sada

'*******************************************************************************
'Variveis globais
Dim EstadoP As Byte                     'sentido atual do porto - 1 aberto, 2 fechado
Dim Cod As Byte                         'aviso de cdigo vlido
Dim avisoH as Byte                      'aviso para entrada/sada de veculos
Dim aux1 as byte                        'vars para uso na decodificao dos cdigos enviados
Dim aux2 as byte
Dim aux3 as byte

'*******************************************************************************
'Configura Timer 1 - usado para lmpadas de aviso
'Timer, modo assincrono, preescale 128
'clock externo 10 MHz
'10MHz / 1024(prescale) = 9.7656kHz => T = 1 / 9.7656kHz = 102.4 us
'102.4 us x 48828 = 500 ms
'entao haver uma int a cada 500 ms
Config Timer1 = Timer , Prescale = 1024
Timer1 = 65535 - 4882                   'valor inicial da contagem

'*******************************************************************************
'Liga pinos de I/O's a "nomes" mais fceis ao entendimento humano ;-)
LP1 Alias Portb.0                       'sada para LP1 - aviso sada de veculo
LP2 Alias Portb.1                       'sada para LP2 - aviso sada de veculo
LPH Alias Portb.2                       'sada para LPH - aviso problemas na garagem
CD1 Alias Pinb.4                        'cdigo recebido - bit 0
CD2 Alias PINB.5                        '                  bit 1
CD3 Alias PINB.6                        '                  bit 2
CD4 Alias PINB.7                        '                  bit 3

VT Alias Pind.2                         'cdigo vlido presente
S1 Alias Pind.3                         'pino de entrada para chave 1 - fim de curso aberto
S2 Alias Pind.4                         'pino de entrada para chave 2 - fim de curso fechado
RL1 Alias Portd.5                       'rele liga motor
RL2 Alias Portd.6                       'rele inverte motor

'*******************************************************************************
'Declarao das subrotinas
Declare Sub Check_cod()                 'verifica cdigo
Declare Sub Off_motor()                 'desliga motor
Declare Sub decode()                    'decodifica e autoriza mover porto
Declare Sub abre_portao()               'abre porto
Declare Sub fecha_portao()              'fecha porto

'*******************************************************************************
'Incio do programa

'zera vars
EstadoP = 0                             'porto ???                    'zera vars
Cod = 0                                 'nenhum cdigo vlido
avisoH = 0                              'aviso desativado
Cod = 0

'l cdigos
Restore Cods                            'aponta para ponteiro dos dados
Read aux2                               'L cdigo normal
Read aux3                               'L cdigo help

'desliga pinos de I/O
Reset LP1                               'LP1 desligada
Reset LP2                               'LP2 desligada
Reset LPh                               'LPH desligada
Reset RL1                               'rele 1 desligado
Reset RL2                               'rele 2 desligado

Waitms 500                              'espera iniciar

'verifica se porto est aberto ou fechado
If S1 = 0 Then                          'se final de curso aberto, indica porto aberto
   EstadoP = 1
End If

If S2 = 0 Then                          'se final de curso fechado, indica porto fechado
   EstadoP = 2
End If

'porto no est nem aberto, nem fechado... inicia como fechado!
If EstadoP = 0 Then                     'porto no esta aberto nem fechado,
   EstadoP = 1                          'indica porto aberto
   call fecha_portao()                  'fecha o portao
End If

On TIMER1 MyISR_T1                      'quando houver uma interrupo do timer 1, desvia para MyISR_T1
Enable TIMER1                           'habilita timer 1
Enable INTERRUPTS                       'habilita interrupes

'*******************************************************************************
'Lao principal
'Algumas operes do programa operam via interrupes (timer1 e int0)
Do

   if VT = 1 then                       'espera cdigo vlido
      call decode()                     'decodifica
   end if

Loop

'*******************************************************************************
'Label para tratamento da interrupo do Timer1
'Usado para alerta de sada/entrada de veculo
'Tempo de acionamento em 500ms
MyISR_T1:

   if avisoH = 1 then                   'se aviso est autorizado
      toggle LP1                        'troca estado da Lmpada 1 e 2
      toggle LP2
   end if

   Timer1 = 65535 - 4882                'retorna valor para nova contagem
   return                               'gera um RETI
   return                               'gera um RET

'*******************************************************************************
' Subrotina para decodificar e autorizar operaao do porto
sub decode()

   call check_cod()                     'verifica cdigo
   if Cod = 1 or Cod = 2 then           'codigo normal
      If EstadoP = 1 then               'porto aberto?
         call fecha_portao()            'ento fecha porto
      else
         If EstadoP = 2 then            'porto fechado?
            call abre_portao()          'ento abre porto
         end if
      end if
   end if

end sub

'*******************************************************************************
'Subrotina para verificar cdigo
Sub Check_cod()

   waitms 50                            'espera at codigo estabilizar
   aux1 = 0                             'zera var

   Restore Cods                         'aponta para ponteiro dos dados
   Read aux2                            'L cdigo normal
   Read aux3                            'L cdigo help

   if CD1 = 1 then                      'verifica estado dos bits e configura var de acordo
      aux1 = aux1 + 1
   end if

   if CD2 = 1 then
      aux1 = aux1 + 2
   end if

   if CD3 = 1 then
      aux1 = aux1 + 4
   end if

   if CD4 = 1 then
      aux1 = aux1 + 8
   end if

   if aux1 = aux2 then                  'se igual, avisa que tudo est normal
      cod = 1
   else
      if aux1 = aux3 then               'se igual, avisa que h problemas
         cod = 2
      else
         cod = 0                        'se diferente de ambos, ignora
      end if
   end if


End Sub

'*******************************************************************************
'Subrotina para abrir o porto
sub abre_portao()

ini_abre:
   if VT = 1 then                       'se ainda ativo
      goto ini_abre                     'segura para evitar repique
   endif

   if Cod = 2 then                      'se  sinal de alerta
      set LPH                           'liga sinal de alerta
   else
      reset LPH                         's desliga num prximo cdigo normal
   end if

   cod = 0                              'limpa autorizao

   set LP1                              'configura lampadas
   reset LP2
   avisoH = 1                           'hablita piscar

   call off_motor()                     'pare os motores

   reset RL2                            'configura sentido
   waitms 10                            'aguarda rele ligar
   set RL1                              'liga rele motor

   waitms 500                           'debounce para operaao motor

cont_abre:                              'label
   if S1 = 0 then                       'chave fim de curso - porto est aberto?
      call off_motor()                  'desliga motor
      EstadoP = 1                       'indica porto aberto
   else
      if VT = 1 then                    'pode ser pedido do usurio para futura inverso
         call check_cod()               'verifica cdigo
         if Cod <> 0 then               'se  valido
            call off_motor()            'pare os motores
            cod = 0
            goto fim_abre               'sai do loop
         end if
      end if
      goto cont_abre                    'ainda no abriu totalmente, continua at abrir
   end if
fim_abre:
end sub

'*******************************************************************************
'Subrotina para fechar o porto
sub fecha_portao()

ini_fecha:
   if VT = 1 then                       'se ainda ativo
      goto ini_fecha                    'segura para evitar repique
   endif

   if Cod = 2 then                      'se  sinal de alerta
      set LPH                           'liga sinal de alerta
   else
      reset LPH                         's desliga num prximo cdigo normal
   end if

   cod = 0                              'limpa autorizao

   set LP1                              'configura lampadas
   reset LP2
   avisoH = 1                           'hablita piscar

   call off_motor()                     'pare os motores

   set RL2                              'configura sentido
   waitms 10                            'aguarda rele ligar
   set RL1                              'liga rele motor

   waitms 500                           'debounce para operaao motor

cont_fecha:                             'label
   if S2 = 0 then                       'chave fim de curso - porto est fechado?
      avisoH = 0                        'desabilita piscar
      reset LP1                         'desliga lmpadas
      reset LP2
      EstadoP = 2                       'indica porto fechado
      call off_motor()                  'desliga motor
   else
      if VT = 1 then                    'pode ser pedido do usurio para futura inverso
         call check_cod()               'verifica cdigo
         if Cod <> 0 then               'se  valido
            call off_motor()            'pare os motores
            cod = 0
            goto fim_fecha              'sai do loop
         end if
      end if
      goto cont_fecha                   'ainda no fechou totalmente, continua at fechar
   end if
fim_fecha:
end sub

'*******************************************************************************
'Subrotina para desligar motor
Sub Off_motor()

   Reset RL1                            'desliga motor
   waitms 10                            'espera rel
   Reset RL2                            'desliga RL2
   wait 1                               'espera para evitar tranco no motor

End Sub

'*******************************************************************************
end                                     'fim do programa

'*******************************************************************************
'Cdigos armazenados na FLASH (rea de programa) do microcontrolador
'Troque os valores de acordo com o desejado
'Veja o artigo para saber mais sobre os valores possveis
Cods:
Data 9 , 5